${HEADER}

'''
Created on ${DATE_TIME}

@author: ${AUTHOR}
'''


from ..common import version 

version.getInstance().submitRevision("$Revision: 131 $")

class Settings():
    '''
    Represents the settings for Gnatirac
    '''
   
    def __init__(self):
        pass
